#!/usr/bin/perl -w
# MIT License
# Author: timeless
use strict;

my @localepaths = </usr/lib/locale/*/LC_IDENTIFICATION>;
my %localemap = ();
my %localenamemap = ();
for my $localepath (@localepaths) {
 $localepath =~ m!/usr/lib/locale/(.*)/LC_IDENTIFICATION!;
 my $locale = $1;
 local $/ = undef;
 open LOCALE, "<", $localepath || next;
  my $line = <LOCALE>;
 close LOCALE;
 $line =~ s/([\x00-\x1f])/sprintf("%%%02x",(unpack("c",$1)))/ge;
 if ($line =~ /%01%00%00(\w{3,}.*?)%00/) {
  $localemap{$locale} = $1;
  $localenamemap{$1} = $locale;
 };
}

my $activelocale;
my @defaultlocales = qw(
 en_US.utf8
 en_GB.utf8
);

if (open LOCALE, "<", $ENV{HOME}."/.osso/current-locale") {
 my $locale = <LOCALE>;
 close LOCALE;
 $locale =~ s/[\n\r]+//g;
 unshift @defaultlocales, $locale;
}
foreach my $locale (@defaultlocales) {
 next unless defined $localemap{$locale};
 $activelocale = $locale;
 last; 
}

my @zenityargs = qw(--list --radiolist --title);
push @zenityargs,"Select locale", "--text", "Change locale", qw(--column= --column=Locale);

for my $localename (sort keys %localenamemap) {
 my $locale = $localenamemap{$localename};
 push @zenityargs, ($locale eq $activelocale ? "TRUE" : "FALSE"), $localename;
};

open (ZENITY, "-|", "/usr/bin/zenity", @zenityargs);
 my $newlocale = <ZENITY>;
close ZENITY;
exit 1 if ($?>>8);

$newlocale =~ s/[\n\r]+//g;
open LOCALE, ">", $ENV{HOME}."/.osso/current-locale";
 print LOCALE $localenamemap{$newlocale};
close LOCALE;
exit 0
